"""Top-level package for blacline."""

__author__ = """Muniappan Puabi"""
__email__ = '95000am@gmail.com'

from .schedule.basic_schedule import basic_schedule
from .schedule.allocation import allocate_buses